@@ -3,15 +3,21 @@ import logging
 import urllib.parse
 from http.server import HTTPServer
 from ipaddress import IPv4Address, IPv4Network
-from os import path
+from os import chdir, path
 from queue import Empty
 from socketserver import ThreadingMixIn
+from sys import version_info as pyversion
 from threading import Thread
 
 import ifaddr
 from RangeHTTPServer import RangeRequestHandler
 
-from .utils import error_and_exit, set_speaker_playing_local_file, set_sigterm
+from .utils import (
+    error_and_exit,
+    event_unsubscribe,
+    set_sigterm,
+    set_speaker_playing_local_file,
+)
 
 # The HTTP server port range to use
 PORT_START = 54000
@@ -19,6 +25,8 @@ PORT_END = 54099
 
 SUPPORTED_TYPES = ["MP3", "M4A", "MP4", "FLAC", "OGG", "WMA", "WAV", "AAC"]
 
+PY37PLUS = True if pyversion.major >= 3 and pyversion.minor >= 7 else False
+
 
 class ThreadedHTTPServer(ThreadingMixIn, HTTPServer):
     """Handle requests in separate threads.
@@ -29,10 +37,26 @@ class ThreadedHTTPServer(ThreadingMixIn, HTTPServer):
 
 
 class MyHTTPHandler(RangeRequestHandler):
-    def __init__(self, *args, filename=None, speaker_ips=None, **kwargs):
-        self.filename = filename
-        self.speaker_ips = speaker_ips
-        super().__init__(*args, **kwargs)
+    # Handle the change to the SimpleHTTPRequestHandler __init__() in Python 3.7+
+    if PY37PLUS:
+
+        def __init__(self, *args, filename=None, speaker_ips=None, **kwargs):
+            self.filename = filename
+            self.speaker_ips = speaker_ips
+            super().__init__(*args, **kwargs)
+
+    else:
+
+        def __init__(
+            self, *args, filename=None, speaker_ips=None, directory="", **kwargs
+        ):
+            self.filename = filename
+            self.speaker_ips = speaker_ips
+            try:
+                chdir(directory)
+            except:
+                pass
+            super().__init__(*args, **kwargs)
 
     def do_GET(self):
         logging.info("Get request received by HTTP server")
@@ -108,7 +132,7 @@ def get_server_ip(speaker):
         return None
 
 
-def wait_until_stopped(speaker, aac_file=False):
+def wait_until_stopped(speaker, uri, aac_file=False):
     sub = speaker.avTransport.subscribe(auto_renew=True)
     # Includes a hack for AAC files, which would be played in a repeat loop.
     # The speaker never goes into a 'STOPPED' state, so we have
@@ -122,13 +146,26 @@ def wait_until_stopped(speaker, aac_file=False):
                     event.variables["transport_state"]
                 )
             )
+
+            # In case there's no STOPPED state event, check that the expected URI
+            # is still playing
+            try:
+                current_uri = event.variables["current_track_meta_data"].get_uri()
+            except:
+                # Can only call get_uri() on certain datatypes
+                current_uri = ""
+            if current_uri != uri:
+                logging.info("Playback URI changed: exit event wait loop")
+                event_unsubscribe(sub)
+                return True
+
             # Special case for AAC files
             if aac_file:
                 if event.variables["transport_state"] == "TRANSITIONING":
                     logging.info("Transitioning event received")
                     if has_played:
                         logging.info("AAC: transition event indicating end of track")
-                        sub.unsubscribe()
+                        event_unsubscribe(sub)
                         speaker.stop()
                         return True
                     else:
@@ -136,11 +173,13 @@ def wait_until_stopped(speaker, aac_file=False):
                 if event.variables["transport_state"] == "PLAYING":
                     has_played = True
                     logging.info("AAC: has_played set to True")
+
             # General case for other file types. Note that pausing (PAUSED_PLAYBACK)
-            # does not terminate the loop.
+            # does not terminate the loop, to allow playback to be resumed
             if event.variables["transport_state"] == "STOPPED":
-                sub.unsubscribe()
+                event_unsubscribe(sub)
                 return True
+
         except Empty:
             pass
 
@@ -214,11 +253,13 @@ def play_local_file(speaker, pathname):
 
     logging.info("Waiting for playback to stop")
     # SIGTERM enablement experiment for AAC files only
-    if aac_file:
+    # Also needed for Python < 3.7
+    if aac_file or not PY37PLUS:
         set_sigterm(True)
-    wait_until_stopped(speaker, aac_file=aac_file)
-    if aac_file:
+        wait_until_stopped(speaker, uri, aac_file=aac_file)
         set_sigterm(False)
+    else:
+        wait_until_stopped(speaker, uri, aac_file=aac_file)
 
     logging.info("Playback stopped ... terminating web server")
     httpd.shutdown()
